package com.bcxin.rbac.domain.repository;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.bcxin.Infrastructures.Pageable;
import com.bcxin.Infrastructures.enums.OccupationType;
import com.bcxin.Infrastructures.exceptions.NotSupportTenantException;
import com.bcxin.Infrastructures.utils.AuthUtil;
import com.bcxin.rbac.domain.RbacCustomDbReader;
import com.bcxin.rbac.domain.dtos.custom.*;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.SQLQuery;
import org.hibernate.transform.Transformers;
import org.springframework.stereotype.Repository;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import java.util.*;

@Repository
@AllArgsConstructor
public class RbacCustomDbReaderImpl implements RbacCustomDbReader {
    private final EntityManager entityManager;

    /**
     * description：查询企业自定义角色列表
     * author：linchunpeng
     * date：2024/2/1
     */
    @Override
    public List<RbacCustomRoleDto> findOrgRoleList(String organizationId, String roleName, String employeeId) {
        Map<String, String> params = new HashMap<>();
        String sql = "select new com.bcxin.rbac.domain.dtos.custom.RbacCustomRoleDto(" +
                "t.id, t.roleName, t.roleType, t.remark, t.seq, t.createTime, t.updateTime" +
                ") " +
                " from RbacCustomRoleEntity t ";
        if (StringUtils.isNotBlank(employeeId)) {
            sql += " left join RbacCustomRoleUserEntity u on u.rbacRoleId = t.id ";
        }
        sql += " where t.organizationId = :organizationId ";
        params.put("organizationId", organizationId);

        if (StringUtils.isNotBlank(employeeId)) {
            sql += " and u.tenantEmployeeId = :employeeId ";
            params.put("employeeId", employeeId);
        }

        if (StringUtils.isNotBlank(roleName)) {
            sql += " and t.roleName like :roleName ";
            params.put("roleName", "%"+roleName+"%");
        }
        sql += " order by t.seq desc ";
        TypedQuery<RbacCustomRoleDto> query = this.entityManager.createQuery(sql, RbacCustomRoleDto.class);
        for (String key : params.keySet()) {
            query.setParameter(key, params.get(key));
        }
        return query.getResultList();
    }

    /**
     * description：分页查询用户职员列表
     * author：linchunpeng
     * date：2024/2/3
     */
    @Override
    public Pageable<RbacUserDto> getRbacUserDtoList(RbacUserSearchDto searchDto) {
        Map<String, Object> parameters = new HashMap<>();
        String sqlStr = "select e.id as employeeId, e.occupation_type as occupationType, t.name as userName, " +
                "t.telephone as mobile, d.id as departId, d.name as departName ";
        if (searchDto.getRoleId() != null) {
            sqlStr += ", ru.create_time as addRoleTime ";
        }
        sqlStr += "from tenant_employees e " +
                "left join tenant_users t on e.tenant_user_id = t.id " +
                "left join tenant_departments d on e.department_id = d.id " +
                "left join tenant_user_credentials c on t.id = c.tenant_user_id and c.selected = 1 ";
        if (searchDto.getRoleId() != null) {
            sqlStr += "left join rbac_custom_role_user ru on e.id = ru.tenant_employee_id ";
        }

        sqlStr += " where e.status = 0 and e.organization_id = :organizationId ";
        parameters.put("organizationId", searchDto.getOrganizationId());


        String sqlCount = "select count(1) from tenant_employees e " +
                "left join tenant_users t on e.tenant_user_id = t.id " +
                "left join tenant_departments d on e.department_id = d.id " +
                "left join tenant_user_credentials c on t.id = c.tenant_user_id and c.selected = 1 ";

        if (searchDto.getRoleId() != null) {
            sqlCount += "left join rbac_custom_role_user ru on e.id = ru.tenant_employee_id ";
        }
        sqlCount += " where e.status = 0 and e.organization_id = :organizationId ";

        if (searchDto.getRoleId() != null) {
            sqlStr += " and ru.rbac_role_id = :roleId ";
            sqlCount += " and ru.rbac_role_id = :roleId ";
            parameters.put("roleId", searchDto.getRoleId());
        }
        if (StringUtils.isNotBlank(searchDto.getKeyWord())) {
            sqlStr += " and (t.name like :keyword or t.telephone like :keyword or c.number like :keyword) ";
            sqlCount += " and (t.name like :keyword or t.telephone like :keyword or c.number like :keyword) ";
            parameters.put("keyword", "%" + searchDto.getKeyWord() + "%");
        }
        if (StringUtils.isNotBlank(searchDto.getDepartId())) {
            sqlStr += " and d.id = :departId ";
            sqlCount += " and d.id = :departId ";
            parameters.put("departId", searchDto.getDepartId());
        }
        if (searchDto.getOccupationType() != null) {
            sqlStr += " and e.occupation_type =:occupationType ";
            sqlCount += " and e.occupation_type =:occupationType ";
            parameters.put("occupationType", searchDto.getOccupationType().ordinal());
        }

        Query query = this.entityManager.createNativeQuery(sqlStr);
        Query cquery = this.entityManager.createNativeQuery(sqlCount);

        for (String key : parameters.keySet()) {
            query.setParameter(key, parameters.get(key));
            cquery.setParameter(key, parameters.get(key));
        }
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(RbacUserDto.class));
        query.setFirstResult(searchDto.getSkip()).setMaxResults(searchDto.getPageSize());
        List<RbacUserDto> resultList = query.getResultList();
        int sum = Integer.valueOf(cquery.getResultList().get(0).toString());
        return Pageable.create(searchDto.getPageIndex(), searchDto.getPageSize(), sum, resultList);
    }

    /**
     * description：查询企业授权的app列表
     * author：linchunpeng
     * date：2024/2/3
     */
    @Override
    public List<RbacPermitAppDto> findOrgAppList(String organizationId) {
        String findAppCodeListSql = "select BIND_APPLICATIONS from t_domain where ID = :organizationId ";
        Query findAppCodeListQuery = this.entityManager.createNativeQuery(findAppCodeListSql);
        findAppCodeListQuery.setParameter("organizationId", organizationId);
        String bindApplications = findAppCodeListQuery.getResultList().get(0).toString();
        List<String> authAppCodeList = new ArrayList<>();
        if (StringUtils.isNotBlank(bindApplications)) {
            authAppCodeList.addAll(JSONObject.parseArray(bindApplications, String.class));
        }
        authAppCodeList.add("__zidingyiyingyongPTJCGN");

        Map<String, Object> params = new HashMap<>();
        String sql = "select new com.bcxin.rbac.domain.dtos.custom.RbacPermitAppDto(" +
                "t.id, t.appCode, t.appName, t.appLogo, t.category" +
                ") " +
                " from RbacPermitAppEntity t " +
                " where t.appCode in :authAppCodeList " +
                " order by t.categorySeq, t.seq asc ";
        params.put("authAppCodeList", authAppCodeList);
        TypedQuery<RbacPermitAppDto> query = this.entityManager.createQuery(sql, RbacPermitAppDto.class);
        for (String key : params.keySet()) {
            query.setParameter(key, params.get(key));
        }
        return query.getResultList();
    }

    /**
     * description：查询应用的操作项列表
     * author：linchunpeng
     * date：2024/2/3
     */
    @Override
    public List<RbacPermitOptionDto> findAppOptionList(Long appId) {
        Map<String, Object> params = new HashMap<>();
        String sql = "select new com.bcxin.rbac.domain.dtos.custom.RbacPermitOptionDto(" +
                "t.id, t.permitAppId, t.appCode, t.category, t.optionType, t.optionCode, t.optionName" +
                ") " +
                " from RbacPermitOptionEntity t " +
                " where t.permitAppId = :appId " +
                " order by t.seq asc ";
        params.put("appId", appId);
        TypedQuery<RbacPermitOptionDto> query = this.entityManager.createQuery(sql, RbacPermitOptionDto.class);
        for (String key : params.keySet()) {
            query.setParameter(key, params.get(key));
        }
        return query.getResultList();
    }


    /**
     * description：通过组织id、姓名列表，查询职员
     * author：linchunpeng
     * date：2024/2/6
     */
    @Override
    public List<RbacUserDto> getRbacUserDtoList(String organizationId, List<String> names) {
        Map<String, Object> parameters = new HashMap<>();
        String sqlStr = "select e.id as employeeId, t.name as userName, t.telephone as mobile " +
                " from tenant_employees e " +
                " left join tenant_users t on e.tenant_user_id = t.id " +
                " where e.status = 0 and e.organization_id = :organizationId and t.name in (:names) ";
        parameters.put("organizationId", organizationId);
        parameters.put("names", names);

        Query query = this.entityManager.createNativeQuery(sqlStr);

        for (String key : parameters.keySet()) {
            query.setParameter(key, parameters.get(key));
        }
        query.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(RbacUserDto.class));
        List<RbacUserDto> resultList = query.getResultList();
        return CollectionUtil.isNotEmpty(resultList) ? resultList : new ArrayList<>();
    }
}
